require 'eagleclaw/browser'
require 'eagleclaw/callbacks'
require 'eagleclaw/formatters'
require 'eagleclaw/scrapers'
require 'eagleclaw/string'
require 'eagleclaw/xml'

module EagleClaw
  class Scraper
    class << self
      include ::EagleClaw::Callbacks
      
      attr_accessor :properties
      
      ##
      # Define a pre-processor to run in a certain context.
      #
      # @param [Symbol] context either `:each` or `:all`.
      # @param [optional, Symbol] meth name of method to call.
      # @return [nil]
      #
      # @overload before(:each, :method_name)
      #   Run the given method before each component of the run.
      # @overload before(:all, :method_name)
      #   Run the given method before the run itself.
      # @overload before(:each, &block)
      #   Run the given block (using `instance_eval`) before each component of
      #   the run.
      # @overload before(:all, &block)
      #   Run the given block (using `instance_eval`) before the run itself.
      #
      # @example Fetch a page before the run
      #   before(:all) do
      #     agent.get("http://google.com/")
      #   end
      # 
      # @example Reset the page before each component of the run
      #   before(:each) do
      #     agent.get("http://google.com/")
      #   end
      #
      def before(context, meth = nil, &block)
        register([:before, context], meth, &block)
      end
      
      ##
      # Define a post-processor to run in a certain context.
      # 
      # @param [Symbol] context either `:each` or `:all`.
      # @param [optional, Symbol] meth name of method to call.
      # @return [nil]
      # 
      # @overload after(:each, :method_name)
      #   Run the given method after each component of the run.
      # @overload after(:all, :method_name)
      #   Run the given method after the run itself.
      # @overload after(:each, &block)
      #   Run the given block (using `instance_eval`) after each component of
      #   the run.
      # @overload after(:all, &block)
      #   Run the given block (using `instance_eval`) after the entire run.
      #
      # @see before
      #
      def after(context, meth = nil, &block)
        register([:after, context], meth, &block)
      end
      
      def prop(prop_name, meth = nil, &block)
        (@properties ||= []) << prop_name.to_sym
        register([:property, prop_name.to_sym], meth, &block)
      end
    end # class << self (Scraper)
    
    include Browser
    
    ##
    # A `Hash` which holds data collected during a run.
    #
    # @see #initialize
    # @see #reset
    attr_accessor :data
    
    ##
    # An `Array` which collects 
    attr_accessor :problems
    
    ##
    # Create a new {Scraper} instance.
    #
    # By default, just sets {#data @data} and {#problems @problems} to empty
    # `Array`s.
    #
    def initialize
      @data = []
      @problems = []
    end
    
    ##
    # Reset this scraper instance's state.
    # 
    # The default version of this method just clears {#data @data} and
    # {#problems @problems}.
    # 
    # @return [nil]
    # @abstract Subclass and extend to reset the scraper state.
    #
    def reset
      data.clear
      problems.clear
    end
    
    ##
    # Run the scraper.
    #
    # Operating procedure:
    #
    # 1.  Run {Scraper.before before(:all)} blocks.
    # 2.  For each property (defined with {Scraper.prop prop(:prop_name)}):
    #     1.  Run {Scraper.before before(:each)} blocks.
    #     2.  Run the property itself.
    #     3.  Runs {Scraper.after after(:each)} blocks.
    # 3.  Runs {Scraper.after after(:all)} blocks.
    # 4.  Return {#data data}.
    #
    # @see #reset
    # @see #data
    def run
      self.class.run_callbacks([:before, :all], self)
      self.class.properties.each do |property|
        self.class.run_callbacks([:before, :each], self)
        self.class.run_callbacks([:property, property], self)
        self.class.run_callbacks([:after, :each], self)
      end
      self.class.run_callbacks([:after, :all], self)
      data
    end
  end # class Scraper
end
